#! /usr/bin/env python

import logging, logging.config, os
from celery import Celery
from celery.signals import setup_logging
from django.conf import settings

LOG = logging.getLogger (__name__)
DEFAULT_LOGCONF = "/etc/qeventlog/logging.conf"

os.environ.setdefault ("DJANGO_SETTINGS_MODULE", "qeventlog.settings")
app = Celery ("qeventlog", include = ["qeventlog.tasks",])
app.config_from_object ("django.conf:settings")
app.autodiscover_tasks (lambda: settings.INSTALLED_APPS)
app.set_current ()

@setup_logging.connect
# @logtool.log_call
def setup_logging_handler (**kwargs): # pylint: disable=W0613
  logging.config.fileConfig (DEFAULT_LOGCONF,
                               disable_existing_loggers = False)

if __name__ == "__main__":
  app.start()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-08 21:22
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='QChildTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(db_index=True)),
                ('timestamp', models.DecimalField(blank=True, db_index=True, decimal_places=6, max_digits=30, null=True)),
                ('parent', models.UUIDField(db_index=True)),
                ('child', models.UUIDField(db_index=True)),
            ],
            options={
                'ordering': ['created', 'timestamp'],
            },
        ),
        migrations.CreateModel(
            name='QEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(db_index=True)),
                ('timestamp', models.DecimalField(blank=True, db_index=True, decimal_places=6, max_digits=30, null=True)),
                ('event', models.CharField(db_index=True, max_length=64)),
                ('task_id', models.UUIDField(db_index=True)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, db_index=True, null=True)),
            ],
            options={
                'ordering': ['created'],
            },
        ),
        migrations.CreateModel(
            name='QTaskState',
            fields=[
                ('task_id', models.UUIDField(db_index=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(db_index=True)),
                ('timestamp', models.DecimalField(blank=True, db_index=True, decimal_places=6, max_digits=30, null=True)),
                ('retries', models.IntegerField(blank=True, db_index=True, null=True)),
                ('status', model_utils.fields.StatusField(choices=[(b'before_task_publish', b'before_task_publish'), (b'after_task_publish', b'after_task_publish'), (b'task_prerun', b'task_prerun'), (b'task_retry', b'task_retry'), (b'task_postrun', b'task_postrun'), (b'task_success', b'task_success'), (b'task_failure', b'task_failure'), (b'task_revoked', b'task_revoked')], db_index=True, default=b'before_task_publish', max_length=100, no_check_for_status=True)),
            ],
            options={
                'ordering': ['created'],
            },
        ),
        migrations.AlterIndexTogether(
            name='qtaskstate',
            index_together=set([('created', 'timestamp', 'status')]),
        ),
        migrations.AlterIndexTogether(
            name='qevent',
            index_together=set([('event', 'task_id', 'timestamp')]),
        ),
        migrations.AlterIndexTogether(
            name='qchildtask',
            index_together=set([('parent', 'child')]),
        ),
    ]



#! /usr/bin/env python

import logging, logtool, sys

LOG = logging.getLogger (__name__)

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname (os.path.dirname (__file__))

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "SidewallSleighHoveringEfficientlyInfinitiesReceptiveJudoOuch"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

# Application definition
INSTALLED_APPS = (
  "raven.contrib.django.raven_compat",
  "qeventlog",
)
MIDDLEWARE_CLASSES = ()

# Time for Django
TIME_ZONE = "UTC"
USE_TZ = False

# Django now insists on this
ROOT_URLCONF = "qeventlog.urls"

# It's not always possible to detect connection loss in a timely
# manner using TCP/IP alone, so AMQP defines something called heartbeats
# that's is used both by the client and the broker to detect if a
# connection was closed.
BROKER_HEARTBEAT = 10

# Maximum number of tasks a pool worker process can execute before
# it's replaced with a new one. Default is no limit.
CELERYD_MAX_TASKS_PER_CHILD = 500

# Let"s use SQL for the results?  We need SQL that isn't Django specific.
# CELERY_RESULT_BACKEND = "db+mysql://task:task@localhost/task"
# CELERY_RESULT_BACKEND = "djcelery.backends.database:DatabaseBackend"

# django-celery also ships with a scheduler that stores the schedule
# in the Django database:
#CELERYBEAT_SCHEDULER = "djcelery.schedulers.DatabaseScheduler"

CELERY_RESULT_SERIALIZER = "json"
CELERY_TASK_SERIALIZER = "json"
CELERY_ACCEPT_CONTENT = ["json",]  # Ignore other content

# Need to investigate failure handling.
#def my_on_failure(self, exc, task_id, args, kwargs, einfo):
#    print("Oh no! Task failed: {0!r}".format(exc))
#CELERY_ANNOTATIONS = {"*": {"on_failure": my_on_failure}}

# Number of simultaneous jobs to run on the worker
# No reason for this to not float.
# CELERYD_CONCURRENCY = 1

# Number of jobs for the worker to take off the queue at a time.
CELERYD_PREFETCH_MULTIPLIER = 1

# Task hard time limit in seconds. The worker processing the task will
# be killed and replaced with a new one when this is exceeded.
CELERYD_TASK_TIME_LIMIT = 30

# Task soft time limit in seconds.
# The SoftTimeLimitExceeded exception will be raised when this is
# exceeded. The task can catch this to e.g. clean up before the hard
# time limit comes.
# http://celery.readthedocs.org/en/latest/configuration.html#celeryd-task-soft-time-limit
CELERYD_TASK_SOFT_TIME_LIMIT = 60

# If True the task will report its status as "started" when the task
# is executed by a worker. The default value is False as the normal
# behaviour is to not report that level of granularity. Tasks are either
# pending, finished, or waiting to be retried. Having a "started" state
# can be useful for when there are long running tasks and there is a
# need to report which task is currently running.
CELERY_TRACK_STARTED = True

# Late ack means the task messages will be acknowledged after the task
# has been executed, not just before, which is the default behavior.
CELERY_ACKS_LATE = True

# Keep results forever
# CELERY_TASK_RESULT_EXPIRES = 0  # Never

# If set to True, result messages will be persistent. This means the
# messages will not be lost after a broker restart. The default is for
# the results to be transient.
# CELERY_RESULT_PERSISTENT = True

# Send events so the worker can be monitored by tools like celerymon.
# CELERY_SEND_EVENTS = False

# If enabled, a task-sent event will be sent for every task so tasks
# can be tracked before they are consumed by a worker.
# CELERY_SEND_TASK_SENT_EVENT = True

# Enables error emails.
CELERY_SEND_TASK_ERROR_EMAILS = False

## # This setting can be used to rewrite any task attribute from the
## # configuration. The setting can be a dict, or a list of annotation
## # objects that filter for tasks and return a map of attributes to
## # change.
##
## import logging, logtool
##
## @logtool.log_call (log_level = logging.ERROR)
## def mp_on_failure (self, exc, task_id, args, kwargs, einfo):
##   LOG = logging.getLogger (__name__)
##   logtool.log_fault (exc)
##
## CELERY_ANNOTATIONS = {"*": {"on_failure": mp_on_failure}}

# This option enables so that every worker has a dedicated queue, so
# that tasks can be routed to specific workers.
CELERY_WORKER_DIRECT = True

# The name of the default queue used by .apply_async if the message
# has no route or no custom queue has been specified.
CELERY_DEFAULT_QUEUE = "qeventlog"

CELERY_ROUTES = {
  "qeventlog.tasks.log": {"exchange": "qeventlog", "routing_key": "qeventlog"}
}

# A sequence of modules to import when the worker starts.
CELERY_IMPORTS = ("celery_statsd",)

# https://github.com/ssaw/celery-statsd
STATSD_HOST = "127.0.0.1"
STATSD_PORT = 8125
CELERYD_STATS_PREFIX = "qeventlog."

# How long to wait before trying a job again
FAIL_WAITTIME = 600

# How many times to retry failed jobs
FAIL_RETRYCOUNT = 5

LOGGING = "/etc/qeventlog/logging.conf"
LOGGING_CONFIG = "qeventlog.logs.logging_loader"

EXTERNAL_CONFIG = "/etc/qeventlog/qeventlog.conf"
execfile (EXTERNAL_CONFIG)

DESIRED_VARIABLES = [
  "LOGGING",
]
REQUIRED_VARIABLES = [
  "BROKER_URL",
  "DATABASES",
  "DEBUG",
  "RAVEN_CONFIG",
]

@logtool.log_call
def check_vars (wanted, provided):
  return [var for var in wanted if var not in provided]

missing = check_vars (DESIRED_VARIABLES, vars ())
if missing:
  print >> sys.stderr, "Missing desired configurations: %s" % missing
missing = check_vars (REQUIRED_VARIABLES, vars ())
if missing:
  print >> sys.stderr, "Missing required configurations: %s" % missing
  sys.exit ()



#!/usr/bin/env python

from __future__ import absolute_import
import logging, logtool, raven, sys
from celery import current_app
from celery.exceptions import Retry
from django.conf import settings
import qeventlog.main # pylint: disable=unused-import
from .models import QEvent

LOG = logging.getLogger (__name__)

@logtool.log_call
def sentry_exception (e, request, message = None):
  sentry_tags = {"component": "qeventlog"}
  try:
    sentry = raven.Client (settings.RAVEN_CONFIG["dsn"],
                           auto_log_stacks = True)
    logtool.log_fault (e, message = message)
    data = {
      "job": request,
    }
    if message:
      data["message"] = message
    sentry.extra_context (data)
    if e is not None:
      einfo = sys.exc_info ()
      rc = sentry.captureException (einfo, **sentry_tags)
      del einfo
    else:
      rc = sentry.capture (**sentry_tags)
    LOG.error ("Sentry filed: %s", rc)
  except Exception as ee:
    logtool.log_fault (ee, message = "FAULT: Problem logging exception.")

@logtool.log_call
def retry_handler (task, e):
  try:
    LOG.info ("Retrying.  Attempt: #%s", task.request.retries)
    raise task.retry (exc = e, max_retries = settings.FAIL_RETRYCOUNT,
                      countdown = (settings.FAIL_WAITTIME
                                   * (task.request.retries + 1)))
  except Retry: # Why yes, we're retrying
    raise
  except: # pylint: disable=W0702
    LOG.error ("Max retries reached: %s  GIVING UP!", task.request.retries)
    sentry_exception (e, task.request)
    raise

#
# Tasks
#

@logtool.log_call
@current_app.task (bind = True)
def log (self, date_t, **kwargs):
  try:
    QEvent.record (date_t, **kwargs)
  except Exception as e:
    retry_handler (self, e)



#! /usr/bin/env python

from setuptools import setup, find_packages
import glob, versioneer

setup (
    name = "qeventlog",
    version = versioneer.get_version (),
    description = "Celery job event logger ",
    long_description = file ("README.rst").read (),
    cmdclass = versioneer.get_cmdclass (),
    classifiers = [
      "Development Status :: 4 - Beta",
      "Intended Audience :: Developers",
      "License :: OSI Approved :: "
      + "GNU General Public License v3 or later (GPLv3+)",
      "Topic :: Utilities",
    ],
    keywords = "celery event logger",
    author = "J C Lawrence",
    author_email = "claw@kanga.nu",
    url = "https://github.com/clearclaw/qeventlog",
    license = "GPL v3",
    packages = find_packages (exclude = ["tests",]),
    package_data = {"qeventlog": ["_cfgtool/qeventlog",
                                  "_cfgtool/*.templ",
                                  "_cfgtool/install",],
    },
    data_files = [
        ("/etc/cfgtool/module.d/", ["qeventlog/_cfgtool/qeventlog",]),
        ("/etc/qeventlog", glob.glob ("qeventlog/_cfgtool/*.templ")),
        ("./bin", ["qeventlog/qeventlog_manage.py"]),
    ],
    zip_safe = False,
    install_requires = [line.strip ()
                        for line in file ("requirements.txt").readlines ()],
    entry_points = {
        "console_scripts": [
            "qeventlog = qeventlog.main:main",
        ],
    },
)



#! /usr/bin/env python

import logtool, logging, logging.config

@logtool.log_call
def logging_loader (thing):
  if isinstance (thing, str):
    logging.config.fileConfig (thing, disable_existing_loggers = False)
  elif isinstance (thing, dict):
    logging.config.dictConfig (thing)
  else:
    raise ValueError ("Unknown logging setting type.")
